@@ -4,6 +4,7 @@ from functools import wraps
 
 from sql import Null
 
+from trytond.i18n import gettext
 from trytond.model import ModelSQL, fields
 from trytond.pyson import Eval, Or, Bool
 from trytond import backend
@@ -13,6 +14,8 @@ from trytond.tools.multivalue import migrate_property
 from trytond.modules.company.model import (
     CompanyMultiValueMixin, CompanyValueMixin)
 
+from .exceptions import AccountError, TaxError
+
 __all__ = ['Category', 'CategoryAccount',
     'CategoryCustomerTax', 'CategorySupplierTax',
     'Template', 'Product', 'account_used', 'template_property',
@@ -33,10 +36,10 @@ def account_used(field_name, field_string=None):
             if not account and not Transaction().readonly:
                 field = (
                     self.fields_get([field_string])[field_string]['string'])
-                self.raise_user_error('missing_account', {
-                        'field': field,
-                        'name': self.rec_name,
-                        })
+                raise AccountError(
+                    gettext('account_product.msg_missing_account',
+                        field=field,
+                        name=self.rec_name))
             if account:
                 return account.current()
         return wrapper
@@ -68,7 +71,7 @@ class Category(CompanyMultiValueMixin, metaclass=PoolMeta):
         'product.category.account', 'category', "Accounts")
     account_expense = fields.MultiValue(fields.Many2One('account.account',
             'Account Expense', domain=[
-                ('kind', '=', 'expense'),
+                ('type.expense', '=', True),
                 ('company', '=', Eval('context', {}).get('company', -1)),
                 ],
             states={
@@ -79,7 +82,7 @@ class Category(CompanyMultiValueMixin, metaclass=PoolMeta):
             depends=['account_parent', 'accounting']))
     account_revenue = fields.MultiValue(fields.Many2One('account.account',
             'Account Revenue', domain=[
-                ('kind', '=', 'revenue'),
+                ('type.revenue', '=', True),
                 ('company', '=', Eval('context', {}).get('company', -1)),
                 ],
             states={
@@ -122,18 +125,14 @@ class Category(CompanyMultiValueMixin, metaclass=PoolMeta):
             },
         depends=['taxes_parent', 'accounting'],
         help="The taxes to apply when purchasing products of this category.")
-    customer_taxes_used = fields.Function(fields.One2Many('account.tax', None,
-            'Customer Taxes Used'), 'get_taxes')
-    supplier_taxes_used = fields.Function(fields.One2Many('account.tax', None,
-            'Supplier Taxes Used'), 'get_taxes')
+    customer_taxes_used = fields.Function(fields.Many2Many(
+            'account.tax', None, None, "Customer Taxes Used"), 'get_taxes')
+    supplier_taxes_used = fields.Function(fields.Many2Many(
+            'account.tax', None, None, "Supplier Taxes Used"), 'get_taxes')
 
     @classmethod
     def __setup__(cls):
         super(Category, cls).__setup__()
-        cls._error_messages.update({
-            'missing_account': ('There is no '
-                    '"%(field)s" defined on the category "%(name)s"'),
-            })
         cls.parent.domain = [
             ('accounting', '=', Eval('accounting', False)),
             cls.parent.domain or []]
@@ -184,7 +183,7 @@ class Category(CompanyMultiValueMixin, metaclass=PoolMeta):
         else:
             return [x.id for x in getattr(self, name[:-5])]
 
-    @fields.depends('parent', '_parent_parent.accounting', 'accounting')
+    @fields.depends('parent', 'accounting')
     def on_change_with_accounting(self):
         if self.parent:
             return self.parent.accounting
@@ -231,14 +230,14 @@ class CategoryAccount(ModelSQL, CompanyValueMixin):
     account_expense = fields.Many2One(
         'account.account', "Account Expense",
         domain=[
-            ('kind', '=', 'expense'),
+            ('type.expense', '=', True),
             ('company', '=', Eval('company', -1)),
             ],
         depends=['company'])
     account_revenue = fields.Many2One(
         'account.account', "Account Revenue",
         domain=[
-            ('kind', '=', 'revenue'),
+            ('type.revenue', '=', True),
             ('company', '=', Eval('company', -1)),
             ],
         depends=['company'])
@@ -290,16 +289,6 @@ class Template(CompanyMultiValueMixin, metaclass=PoolMeta):
             ('accounting', '=', True),
             ])
 
-    @classmethod
-    def __setup__(cls):
-        super(Template, cls).__setup__()
-        cls._error_messages.update({
-                'missing_account': ('There is no '
-                    '"%(field)s" defined on the product "%(name)s"'),
-                'missing_taxes': ('There is no account category defined '
-                    'on the product "%(name)s"'),
-                })
-
     def get_account(self, name, **pattern):
         if self.account_category:
             return self.account_category.get_account(name, **pattern)
@@ -330,9 +319,9 @@ class Template(CompanyMultiValueMixin, metaclass=PoolMeta):
             if Transaction().readonly:
                 taxes = []
             else:
-                self.raise_user_error('missing_taxes', {
-                        'name': self.rec_name,
-                        })
+                raise TaxError(
+                    gettext('account_product.msg_missing_taxes',
+                        name=self.rec_name))
         return taxes
 
     @property
@@ -347,9 +336,9 @@ class Template(CompanyMultiValueMixin, metaclass=PoolMeta):
             if Transaction().readonly:
                 taxes = []
             else:
-                self.raise_user_error('missing_taxes', {
-                        'name': self.rec_name,
-                        })
+                raise TaxError(
+                    gettext('account_product.msg_missing_taxes',
+                        name=self.rec_name))
         return taxes
 
 
